UnbreakableSegmentUnderConstruction

Struct UnbreakableSegmentUnderConstruction 

struct UnbreakableSegmentUnderConstruction {
    inline_size: Au,
    max_block_size: LineBlockSizes,
    line_items: Vec<LineItem>,
    has_content: bool,
    incorporates_trailing_white_space: bool,
    trailing_hangable_white_space: Au,
    trailing_removable_white_space: Au,
}
The current unbreakable segment under construction for an inline formatting context. Items accumulate here until we reach a soft line break opportunity during processing of inline content or we reach the end of the formatting context.

Fields

inline_size: Au

The size of this unbreakable segment in both dimension.

max_block_size: LineBlockSizes

The maximum block size that this segment has. This uses LineBlockSizes instead of a simple value, because the final block size depends on vertical alignment.

line_items: Vec<LineItem>

The LineItems for the segment under construction

has_content: bool

Whether any active linebox has added a glyph or atomic element to this line segment, which indicates that the next run that exceeds the line length can cause a line break.

incorporates_trailing_white_space: bool

Whether or not this segment incorporates the trailing white space that came before it on the line into the line’s content.

trailing_hangable_white_space: Au

The inline size of any trailing hangable white space in this segment.

trailing_removable_white_space: Au

The inline size of any trailing removable white space in this segment.

Implementations

Source

impl UnbreakableSegmentUnderConstruction

fn new() -> Self

fn reset(&mut self)

Reset this segment after its contents have been committed to a line.

fn push_line_item(&mut self, line_item: LineItem)

Push a single line item to this segment.

fn trim_leading_whitespace(&mut self)

Trim whitespace from the beginning of this UnbreakbleSegmentUnderConstruction.

From https://www.w3.org/TR/css-text-3/#white-space-phase-2:

Then, the entire block is rendered. Inlines are laid out, taking bidi reordering into account, and wrapping as specified by the text-wrap property. As each line is laid out,

  1. A sequence of collapsible spaces at the beginning of a line is removed.

This prevents whitespace from being added to the beginning of a line.

fn is_phantom(&self) -> bool

Whether this is segment is phantom. If false, its line box won’t be phantom. https://drafts.csswg.org/css-inline-3/#invisible-line-boxes

fn trailing_inline_box_start_size(&self) -> Au

The size of the inline padding, border, and margin start edge of all trailing opening inline boxes on this segment. In addition, if those inline boxes will clone their box decorations, include the size of the cloned end padding, border, and margin as well.

fn hanging_and_removable_white_space_advance(&self) -> Au

The advance of the whitespace at the end of this segment that would either hang or be trimmed if it was at the end of a line.